@@ -194,6 +194,37 @@ bool BulletURDFImporter::loadURDF(const char* fileName, bool forceFixedBase)
 	if (xml_string.length())
 	{
 			result = m_data->m_urdfParser.loadUrdf(xml_string.c_str(), &loggie, forceFixedBase, (m_data->m_flags & CUF_PARSE_SENSORS));
+
+			if (m_data->m_flags & CUF_IGNORE_VISUAL_SHAPES)
+			{
+				for (int i=0; i < m_data->m_urdfParser.getModel().m_links.size(); i++)
+				{
+					UrdfLink* linkPtr = *m_data->m_urdfParser.getModel().m_links.getAtIndex(i);
+					linkPtr->m_visualArray.clear();
+				}
+			}
+			if (m_data->m_flags & CUF_IGNORE_COLLISION_SHAPES)
+			{
+				for (int i=0; i < m_data->m_urdfParser.getModel().m_links.size(); i++)
+				{
+					UrdfLink* linkPtr = *m_data->m_urdfParser.getModel().m_links.getAtIndex(i);
+					linkPtr->m_collisionArray.clear();
+				}
+			}
+			if (m_data->m_urdfParser.getModel().m_rootLinks.size())
+			{
+				if (m_data->m_flags & CUF_MERGE_FIXED_LINKS)
+				{
+					m_data->m_urdfParser.mergeFixedLinks(m_data->m_urdfParser.getModel(), m_data->m_urdfParser.getModel().m_rootLinks[0], &loggie, forceFixedBase, 0);
+					m_data->m_urdfParser.getModel().m_links.clear();
+					m_data->m_urdfParser.getModel().m_joints.clear();
+					m_data->m_urdfParser.recreateModel(m_data->m_urdfParser.getModel(), m_data->m_urdfParser.getModel().m_rootLinks[0], &loggie);
+				}
+				if (m_data->m_flags & CUF_PRINT_URDF_INFO)
+				{
+					m_data->m_urdfParser.printTree(m_data->m_urdfParser.getModel().m_rootLinks[0], &loggie, 0);
+				}
+			}
 	}
 
 	return result;
@@ -605,6 +636,7 @@ btCollisionShape* BulletURDFImporter::convertURDFToCollisionShape(const UrdfColl
 				btVector3 halfExtents(cylRadius, cylRadius, cylHalfLength);
 				btCylinderShapeZ* cylZShape = new btCylinderShapeZ(halfExtents);
 				shape = cylZShape;
+				shape->setMargin(gUrdfDefaultCollisionMargin);
 			}
 			else
 			{
@@ -1252,45 +1284,40 @@ int BulletURDFImporter::convertLinkVisualShapes(int linkIndex, const char* pathP
 
 			convertURDFToVisualShapeInternal(&vis, pathPrefix, localInertiaFrame.inverse() * childTrans, vertices, indices, textures,meshData);
 
-			if (m_data->m_flags&CUF_USE_MATERIAL_COLORS_FROM_MTL)
+			bool mtlOverridesUrdfColor = false;
+			if ((meshData.m_flags & B3_IMPORT_MESH_HAS_RGBA_COLOR) &&
+					(meshData.m_flags & B3_IMPORT_MESH_HAS_SPECULAR_COLOR))
 			{
-				if ((meshData.m_flags & B3_IMPORT_MESH_HAS_RGBA_COLOR) &&
-						(meshData.m_flags & B3_IMPORT_MESH_HAS_SPECULAR_COLOR))
+				mtlOverridesUrdfColor = (m_data->m_flags & CUF_USE_MATERIAL_COLORS_FROM_MTL) != 0;
+				UrdfMaterialColor matCol;
+				if (m_data->m_flags&CUF_USE_MATERIAL_TRANSPARANCY_FROM_MTL)
 				{
-					UrdfMaterialColor matCol;
-
-					if (m_data->m_flags&CUF_USE_MATERIAL_TRANSPARANCY_FROM_MTL)
-					{
-						matCol.m_rgbaColor.setValue(meshData.m_rgbaColor[0],
-									meshData.m_rgbaColor[1],
-									meshData.m_rgbaColor[2],
-									meshData.m_rgbaColor[3]);
-					} else
-					{
-						matCol.m_rgbaColor.setValue(meshData.m_rgbaColor[0],
-									meshData.m_rgbaColor[1],
-									meshData.m_rgbaColor[2],
-									1);
-					}
-
-					matCol.m_specularColor.setValue(meshData.m_specularColor[0],
-						meshData.m_specularColor[1],
-						meshData.m_specularColor[2]);
-					m_data->m_linkColors.insert(linkIndex, matCol);
-				}
-			} else
-			{
-				if (matPtr)
+					matCol.m_rgbaColor.setValue(meshData.m_rgbaColor[0],
+								meshData.m_rgbaColor[1],
+								meshData.m_rgbaColor[2],
+								meshData.m_rgbaColor[3]);
+				} else
 				{
-					UrdfMaterial* const mat = *matPtr;
-					//printf("UrdfMaterial %s, rgba = %f,%f,%f,%f\n",mat->m_name.c_str(),mat->m_rgbaColor[0],mat->m_rgbaColor[1],mat->m_rgbaColor[2],mat->m_rgbaColor[3]);
-					UrdfMaterialColor matCol;
-					matCol.m_rgbaColor = mat->m_matColor.m_rgbaColor;
-					matCol.m_specularColor = mat->m_matColor.m_specularColor;
-					m_data->m_linkColors.insert(linkIndex, matCol);
+					matCol.m_rgbaColor.setValue(meshData.m_rgbaColor[0],
+								meshData.m_rgbaColor[1],
+								meshData.m_rgbaColor[2],
+								1);
 				}
+
+				matCol.m_specularColor.setValue(meshData.m_specularColor[0],
+					meshData.m_specularColor[1],
+					meshData.m_specularColor[2]);
+				m_data->m_linkColors.insert(linkIndex, matCol);
+			}
+			if (matPtr && !mtlOverridesUrdfColor)
+			{
+				UrdfMaterial* const mat = *matPtr;
+				//printf("UrdfMaterial %s, rgba = %f,%f,%f,%f\n",mat->m_name.c_str(),mat->m_rgbaColor[0],mat->m_rgbaColor[1],mat->m_rgbaColor[2],mat->m_rgbaColor[3]);
+				UrdfMaterialColor matCol;
+				matCol.m_rgbaColor = mat->m_matColor.m_rgbaColor;
+				matCol.m_specularColor = mat->m_matColor.m_specularColor;
+				m_data->m_linkColors.insert(linkIndex, matCol);
 			}
-
 		}
 	}
 	if (vertices.size() && indices.size())
@@ -1498,3 +1525,12 @@ class btCompoundShape* BulletURDFImporter::convertLinkCollisionShapes(int linkIn
 
 	return compoundShape;
 }
+
+const struct UrdfModel* BulletURDFImporter::getUrdfModel() const {
+	return &m_data->m_urdfParser.getModel();
+};
+
+const struct UrdfDeformable& BulletURDFImporter::getDeformableModel() const
+{
+	return m_data->m_urdfParser.getDeformable();
+}
