@@ -228,7 +228,7 @@ struct InternalDataRenderer : public GLInstanceRendererInternalData
 
 	b3Vector3 m_lightPos;
 	b3Vector3 m_lightSpecularIntensity;
-
+	float m_shadowmapIntensity;
 	GLuint m_defaultTexturehandle;
 	b3AlignedObjectArray<InternalTextureHandle> m_textureHandles;
 
@@ -257,6 +257,7 @@ struct InternalDataRenderer : public GLInstanceRendererInternalData
 		
 		m_lightPos = b3MakeVector3(-50, 30, 40);
 		m_lightSpecularIntensity.setValue(1, 1, 1);
+		m_shadowmapIntensity = 0.3;
 
 		//clear to zero to make it obvious if the matrix is used uninitialized
 		for (int i = 0; i < 16; i++)
@@ -317,6 +318,8 @@ static GLint useShadow_MVP = 0;
 static GLint useShadow_lightPosIn = 0;
 static GLint useShadow_cameraPositionIn = 0;
 static GLint useShadow_materialShininessIn = 0;
+static GLint useShadow_shadowmapIntensityIn = 0;
+
 
 static GLint useShadow_ProjectionMatrix = 0;
 static GLint useShadow_DepthBiasModelViewMatrix = 0;
@@ -394,6 +397,16 @@ void GLInstancingRenderer::removeAllInstances()
 	m_graphicsInstances.clear();
 	m_data->m_publicGraphicsInstances.exitHandles();
 	m_data->m_publicGraphicsInstances.initHandles();
+
+#if 0
+	//todo: cannot release ALL textures, since some handles are still kept, and it would cause a crash
+	for (int i=0;i<m_data->m_textureHandles.size();i++)
+	{
+		InternalTextureHandle& h = m_data->m_textureHandles[i];
+		glDeleteTextures(1, &h.m_glTexture);
+	}
+	m_data->m_textureHandles.clear();
+#endif
 }
 
 GLInstancingRenderer::~GLInstancingRenderer()
@@ -512,23 +525,26 @@ void GLInstancingRenderer::writeSingleInstanceColorToCPU(const double* color, in
 {
 	b3PublicGraphicsInstance* pg = m_data->m_publicGraphicsInstances.getHandle(srcIndex2);
 	b3Assert(pg);
-	int srcIndex = pg->m_internalInstanceIndex;
-
-	int shapeIndex = pg->m_shapeIndex;
-	b3GraphicsInstance* gfxObj = m_graphicsInstances[shapeIndex];
-	if (color[3] < 1)
+	if (pg)
 	{
-		gfxObj->m_flags |= B3_INSTANCE_TRANSPARANCY;
-	}
-	else
-	{
-		gfxObj->m_flags &= ~B3_INSTANCE_TRANSPARANCY;
-	}
+		int srcIndex = pg->m_internalInstanceIndex;
 
-	m_data->m_instance_colors_ptr[srcIndex * 4 + 0] = float(color[0]);
-	m_data->m_instance_colors_ptr[srcIndex * 4 + 1] = float(color[1]);
-	m_data->m_instance_colors_ptr[srcIndex * 4 + 2] = float(color[2]);
-	m_data->m_instance_colors_ptr[srcIndex * 4 + 3] = float(color[3]);
+		int shapeIndex = pg->m_shapeIndex;
+		b3GraphicsInstance* gfxObj = m_graphicsInstances[shapeIndex];
+		if (color[3] < 1)
+		{
+			gfxObj->m_flags |= B3_INSTANCE_TRANSPARANCY;
+		}
+		else
+		{
+			gfxObj->m_flags &= ~B3_INSTANCE_TRANSPARANCY;
+		}
+
+		m_data->m_instance_colors_ptr[srcIndex * 4 + 0] = float(color[0]);
+		m_data->m_instance_colors_ptr[srcIndex * 4 + 1] = float(color[1]);
+		m_data->m_instance_colors_ptr[srcIndex * 4 + 2] = float(color[2]);
+		m_data->m_instance_colors_ptr[srcIndex * 4 + 3] = float(color[3]);
+	}
 }
 
 void GLInstancingRenderer::writeSingleInstanceColorToCPU(const float* color, int srcIndex2)
@@ -1101,10 +1117,13 @@ void GLInstancingRenderer::activateTexture(int textureIndex)
 	}
 }
 
-void GLInstancingRenderer::updateShape(int shapeIndex, const float* vertices)
+void GLInstancingRenderer::updateShape(int shapeIndex, const float* vertices, int numVertices)
 {
 	b3GraphicsInstance* gfxObj = m_graphicsInstances[shapeIndex];
 	int numvertices = gfxObj->m_numVertices;
+	b3Assert(numvertices == numVertices);
+	if (numvertices != numVertices)
+		return;
 
 	glBindBuffer(GL_ARRAY_BUFFER, m_data->m_vbo);
 	int vertexStrideInBytes = 9 * sizeof(float);
@@ -1299,6 +1318,8 @@ void GLInstancingRenderer::InitShaders()
 	useShadow_lightPosIn = glGetUniformLocation(useShadowMapInstancingShader, "lightPosIn");
 	useShadow_cameraPositionIn = glGetUniformLocation(useShadowMapInstancingShader, "cameraPositionIn");
 	useShadow_materialShininessIn = glGetUniformLocation(useShadowMapInstancingShader, "materialShininessIn");
+	useShadow_shadowmapIntensityIn = glGetUniformLocation(useShadowMapInstancingShader, "shadowmapIntensityIn");
+	
 
 	createShadowMapInstancingShader = gltLoadShaderPair(createShadowMapInstancingVertexShader, createShadowMapInstancingFragmentShader);
 	glLinkProgram(createShadowMapInstancingShader);
@@ -1475,6 +1496,12 @@ void GLInstancingRenderer::setLightPosition(const float lightPos[3])
 	m_data->m_lightPos[2] = lightPos[2];
 }
 
+void GLInstancingRenderer::setShadowMapIntensity(double shadowMapIntensity)
+{
+	m_data->m_shadowmapIntensity = shadowMapIntensity;
+}
+
+
 void GLInstancingRenderer::setShadowMapResolution(int shadowMapResolution)
 {
 	m_data->m_shadowMapWidth = shadowMapResolution;
@@ -1495,6 +1522,11 @@ void GLInstancingRenderer::setLightPosition(const double lightPos[3])
 	m_data->m_lightPos[2] = lightPos[2];
 }
 
+void GLInstancingRenderer::setBackgroundColor(const double rgbBackground[3])
+{
+	glClearColor(rgbBackground[0], rgbBackground[1], rgbBackground[2], 1.f);
+}
+
 void GLInstancingRenderer::setProjectiveTextureMatrices(const float viewMatrix[16], const float projectionMatrix[16])
 {
 	for (int i = 0; i < 16; i++)
@@ -2183,7 +2215,7 @@ void GLInstancingRenderer::renderSceneInternal(int orgRenderMode)
 		//	return;
 		glEnable(GL_CULL_FACE);
 		glCullFace(GL_FRONT);  // Cull back-facing triangles -> draw only front-facing triangles
-
+		glGenerateMipmap(GL_TEXTURE_2D);
 		b3Assert(glGetError() == GL_NO_ERROR);
 	}
 	else
@@ -2346,12 +2378,21 @@ void GLInstancingRenderer::renderSceneInternal(int orgRenderMode)
 				if (gfxObj->m_flags & B3_INSTANCE_TEXTURE)
 				{
 					curBindTexture = m_data->m_textureHandles[gfxObj->m_textureIndex].m_glTexture;
-
-					glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_BASE_LEVEL, 0);
+					glBindTexture(GL_TEXTURE_2D, curBindTexture);
 
 					if (m_data->m_textureHandles[gfxObj->m_textureIndex].m_enableFiltering)
 					{
-						glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
+						glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_BASE_LEVEL, 0);
+						if (renderMode == B3_CREATE_SHADOWMAP_RENDERMODE)
+						{
+							glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
+						}
+						else
+						{
+							glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
+							
+						}
+						
 						glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
 					}
 					else
@@ -2363,13 +2404,12 @@ void GLInstancingRenderer::renderSceneInternal(int orgRenderMode)
 				else
 				{
 					curBindTexture = m_data->m_defaultTexturehandle;
+					glBindTexture(GL_TEXTURE_2D, curBindTexture);
 				}
 
 				//disable lazy evaluation, it just leads to bugs
 				//if (lastBindTexture != curBindTexture)
-				{
-					glBindTexture(GL_TEXTURE_2D, curBindTexture);
-				}
+				
 				//lastBindTexture = curBindTexture;
 
 				b3Assert(glGetError() == GL_NO_ERROR);
@@ -2483,7 +2523,7 @@ void GLInstancingRenderer::renderSceneInternal(int orgRenderMode)
 								glUniform3f(regularLightDirIn, gLightDir[0], gLightDir[1], gLightDir[2]);
 
 								glUniform1i(uniform_texture_diffuse, 0);
-
+								glEnable(GL_MULTISAMPLE);
 								if (gfxObj->m_flags & B3_INSTANCE_TRANSPARANCY)
 								{
 									int instanceId = transparentInstances[i].m_instanceId;
@@ -2511,6 +2551,7 @@ void GLInstancingRenderer::renderSceneInternal(int orgRenderMode)
 							{
 								glUseProgram(createShadowMapInstancingShader);
 								glUniformMatrix4fv(createShadow_depthMVP, 1, false, &depthMVP[0][0]);
+								glEnable(GL_MULTISAMPLE);
 								glDrawElementsInstanced(GL_TRIANGLES, indexCount, GL_UNSIGNED_INT, indexOffset, gfxObj->m_numGraphicsInstances);
 								break;
 							}
@@ -2561,12 +2602,15 @@ void GLInstancingRenderer::renderSceneInternal(int orgRenderMode)
 								m_data->m_activeCamera->getCameraPosition(camPos);
 								glUniform3f(useShadow_cameraPositionIn, camPos[0], camPos[1], camPos[2]);
 								glUniform1f(useShadow_materialShininessIn, gfxObj->m_materialShinyNess);
-
+								glUniform1f(useShadow_shadowmapIntensityIn, m_data->m_shadowmapIntensity);
+								
+								
 								glUniformMatrix4fv(useShadow_DepthBiasModelViewMatrix, 1, false, &depthBiasMVP[0][0]);
 								glActiveTexture(GL_TEXTURE1);
 								glBindTexture(GL_TEXTURE_2D, m_data->m_shadowTexture);
-								glUniform1i(useShadow_shadowMap, 1);
 
+								glUniform1i(useShadow_shadowMap, 1);
+								glEnable(GL_MULTISAMPLE);
 								//sort transparent objects
 
 								//gfxObj->m_instanceOffset
@@ -2724,6 +2768,7 @@ void GLInstancingRenderer::enableShadowMap()
 void GLInstancingRenderer::clearZBuffer()
 {
 	glClear(GL_DEPTH_BUFFER_BIT);
+	glEnable(GL_MULTISAMPLE);
 }
 
 int GLInstancingRenderer::getMaxShapeCapacity() const
