@@ -220,7 +220,7 @@ struct InternalData2
 		if (!m_x11_library)
 		{
 			// TODO: Properly handle this error.
-			fprintf(stderr, "Error opening X11 library %s\n", X11_LIBRARY);
+			fprintf(stderr, "Error opening X11 library %s: %s\n", X11_LIBRARY, dlerror());
 			exit(EXIT_FAILURE);
 		}
 
@@ -566,8 +566,10 @@ void X11OpenGLWindow::enableOpenGL()
 	//Access pthreads as a workaround for a bug in Linux/Ubuntu
 	//See https://bugs.launchpad.net/ubuntu/+source/nvidia-graphics-drivers-319/+bug/1248642
 
+#if !defined(__NetBSD__)
 	int i = pthread_getconcurrency();
 	printf("pthread_getconcurrency()=%d\n", i);
+#endif
 
 	//    const GLubyte* ext = glGetString(GL_EXTENSIONS);
 	//    printf("GL_EXTENSIONS=%s\n", ext);
