@@ -2,8 +2,10 @@ import pybullet as p
 import math
 import time
 dt = 1./240.
+import pybullet_data
 
-p.connect(p.GUI)#SHARED_MEMORY_GUI)
+p.connect(p.GUI)
+p.setAdditionalSearchPath(pybullet_data.getDataPath())
 p.loadURDF("r2d2.urdf",[0,0,1])
 p.loadURDF("plane.urdf")
 p.setGravity(0,0,-10)
@@ -18,4 +20,4 @@ while (1):
   p.configureDebugVisualizer(lightPosition=[radius*math.sin(t),radius*math.cos(t),3])
   
   p.stepSimulation()
-  time.sleep(dt)
\ No newline at end of file
+  time.sleep(dt)
