@@ -1,9 +1,12 @@
 import pybullet as p
 import time
+import pybullet_data
 
 cid = p.connect(p.SHARED_MEMORY)
 if (cid < 0):
   p.connect(p.GUI)
+
+p.setAdditionalSearchPath(pybullet_data.getDataPath())  
 p.loadURDF("plane.urdf")
 kuka = p.loadURDF("kuka_iiwa/model.urdf")
 p.addUserDebugText("tip", [0, 0, 0.1],
