@@ -1,7 +1,10 @@
 import pybullet as p
 import time
 
+import pybullet_data
+
 p.connect(p.GUI)
+p.setAdditionalSearchPath(pybullet_data.getDataPath())
 obUids = p.loadMJCF("mjcf/humanoid.xml")
 humanoid = obUids[1]
 
